        page    60,102
        title   ADI driver for Tektronix 4662 plotter
com	equ	1	;set to 1 for COM1, 2 for COM2
port	equ	400h-100h*com
pltadrs equ     'C'     ;plotter address as set on rear switches
code    segment
        assume cs:code,ds:code
        org     100h
plotter proc

;Main procedure code

begin:  jmp     init

;	Totally	awesome!  We just got an interrupt call	from the
;       vector.  Restore the context we were running in.

intrap:	push	ds		; save user data segment
	push	es		; save user extra segment
        push    si
        push    di
        push    bp              ; save frame pointer
        push    cs              ; get our data segment=control seg
        pop     ds
	mov	css,ss		; save stack segment
	mov	csp,sp		; save stack pointer
	mov	ss,rss		; restore our stack segment
	mov	sp,rsp		; restore our stack pointer
	sti			; enable interrupts
        cld                     ; set the increment bit !

        rol     ax,1            ;form table address
        lea     di,cmdtab-2
	add	di,ax
	jmp	word ptr[di]	;jump thru table

cmdtab	label	byte
        dw      begplt
        dw      endplt
        dw      move
        dw      draw
        dw      insnft          ;new pen
        dw      insnft          ;select speed
        dw      insnft          ;set line type
        dw      insnft          ;pen change
        dw      endplt          ;abort plot

begplt: mov     si,offset strstg
        call    out_stg
        jmp     insnft

endplt: mov     si,offset endstg
        call    out_stg
        jmp     insnft

move:   mov     byte ptr buffer,29
        mov     cksum,204
        mov     di,offset buff2
        jmp     draw2
draw:   mov     cksum,175
        mov     di,offset buffer
draw2:  mov     x,bx
        mov     y,cx
        mov     ax,y            ;first byte
        shr     ax,1            ;y/128
        shr     ax,1
        shr     ax,1
        shr     ax,1
        shr     ax,1
        shr     ax,1
        shr     ax,1
        or      ax,32
        call    sto_chr
        mov     ax,y            ;second byte
        and     ax,3
        shl     ax,1
        shl     ax,1
        mov     bx,x
        and     bx,3
        or      ax,bx
        or      ax,96
        call    sto_chr
        mov     ax,y            ;third byte
        shr     ax,1            ;y/4
        shr     ax,1
        and     ax,31
        or      ax,96
        call    sto_chr
        mov     ax,x            ;fourth byte
        shr     ax,1            ;x/128
        shr     ax,1
        shr     ax,1
        shr     ax,1
        shr     ax,1
        shr     ax,1
        shr     ax,1
        or      ax,32
        call    sto_chr
        mov     ax,x            ;fifth byte
        shr     ax,1            ;x/4
        shr     ax,1
        and     ax,31
        or      ax,64
        call    sto_chr
        mov     al,1bh          ;put block trailer into buffer
        call    put_chr
        mov     al,pltadrs
        call    put_chr
        mov     al,')'
        call    put_chr
        mov     ax,cksum        ;convert checksum to decimal
        mov     dx,0
        div     thous
        or      al,30h          ;convert each digit to ASCII
        call    put_chr         ;and put it in buffer
        mov     ax,dx
        mov     dx,0
        div     hund
        or      al,30h
        call    put_chr
        mov     ax,dx
        mov     dx,0
        div     ten
        or      al,30h
        call    put_chr
        mov     ax,dx
        or      al,30h
        call    put_chr
        mov     al,26
        call    put_chr
outbuf: mov     si,offset plotstg
        call    out_stg         ;transmit block to plotter
        call    get_chr         ;wait for plotter to acknowledge
        cmp     al,'I'          ;did plotter indicate error?
        jz      outbuf          ;if so, transmit block again

insnft: mov     ax,0            ; load AX
	cli			; prevent interrupts
        mov     rss,ss          ; save local stack segment
        mov     rsp,sp          ; and pointer
        mov     ss,css          ; load caller's stack segment
        mov     sp,csp          ; load caller's stack pointer
	sti			; enable interrupts
        pop     bp              ; save base page
        pop     di
        pop     si
        pop     es              ; restore ES
	pop	ds		; restore DS
	iret			; return to caller

out_stg proc    near
out_bsy:mov     dx,port+0fdh    ;check line status
	in	al,dx
	test	al,20h	;is hold register empty?
	jz	out_bsy	;if so, try again until ready
        mov     al,byte ptr [si]        ;retrieve character
	mov	dx,port+0f8h	;transmit character
	out	dx,al
        inc     si
        cmp     al,26
        jne     out_bsy
	ret
out_stg endp

get_chr proc    near
etx1:   mov     dx,port+0fdh    ;check line status
        in      al,dx
etx2:   test    al,1    ;has a character been received?
        jz      etx1
etx3:   mov     dx,port+0f8h    ;if so, get the character
        in      al,dx
get_chr endp

sto_chr proc    near
        call    put_chr
        mov     ah,0                    ;and update checksum
        add     ax,cksum
        cmp     ax,4095         ;modulo 4096 w/end-around carry
        jbe     sto2
        sub     ax,4095
sto2:   mov     cksum,ax
        ret
sto_chr endp

put_chr proc    near
        mov     byte ptr [di],al        ;store character in buffer array
        inc     di
        ret
put_chr endp

strstg  db      1bh,pltadrs,'E',1bh,pltadrs,'H125',1ah
endstg  db      1bh,pltadrs,'F',1ah
plotstg db      1bh,pltadrs,'('
buffer  db      0
buff2   db      125 dup (0)
thous   dw      1000
hund    dw      100
ten     dw      10
x       dw      0
y       dw      0
cksum   dw      0
rss     dw      0               ; save stack segment
rsp     dw      stktop          ; save stack pointer
css     dw      0               ; caller's stack segment
csp     dw      0               ; caller's stack pointer
        dw      20 dup (0)
stktop  dw      0
;Initialization
init:   push    cs              ;set ds=cs
        pop     ds

        mov     dx,port+0fbh    ;initialize COM1
	mov	al,80h	;to 1200,mark,7,1
	out	dx,al
	mov	dx,port+0f8h
	mov	al,60h
	out	dx,al
	mov	dx,port+0f9h
	mov	al,0
	out	dx,al
	mov	dx,port+0fbh
	mov	al,2ah
	out	dx,al
	mov	dx,port+0fch	;set DTR and RTS
	mov	al,3
	out	dx,al
	mov	dx,port+0f9h	;disable interrupts
	mov	al,0
	out	dx,al
        push    cs      ;display segment where loaded
	pop	dx
	lea	di,msg1a
        push    cx
	push	ax
	mov	cx,4
h1:	push	cx
	mov	cl,4
	rol	dx,cl
	mov	al,dl
	and	al,0fh
	cmp	al,0ah
	jge	h2
	add	al,30h
	jmp	h3
h2:	add	al,37h
h3:	mov	cs:[di],al
	inc	di
	pop	cx
	loop	h1
	pop	ax
	pop	cx
        lea     dx,msg1         ;put load message on screen
	mov	ah,9
	int	21h
;
;	Install	resident program and service interrupts
;
        mov     rss,cs          ; save stack segment = control segment
        mov     rsp,offset stktop       ;prepare stack pointer = local stack
        mov     dx,offset intrap
        mov     ax,2578H                ; setup interrupt call
        int     21H
        mov     dx,offset init
        int     27H                     ; terminate and stay resident

msg1    db      0dh,0ah
        db      'Tektronix 4662 ADI plotter driver using COM'
	db	com+30h,0dh,0ah
	db	'By Charles E. Cohn, Austell, Georgia',0dh,0ah
        db      'Version 1.00, plotter address ',pltadrs,0dh,0ah
	db	'Loaded at '
msg1a   db      '0000:0000h',' on interrupt 78h',0dh,0ah,'$'

plotter endp
code    ends
	end	begin
